/************************************************************************
 *   IRC - Internet Relay Chat, m_snotice.c
 *   Copyright (C) 2004 Jeremy Johnston
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   $Id: example_module.c,v 1.10 2004/03/30 00:53:55 leeh Exp $
 */

/* List of ircd includes from ../include/ */
#include "stdinc.h"
#include "modules.h"
#include "hook.h"
#include "client.h"
#include "ircd.h"
#include "send.h"
#include "s_conf.h"
#include "sprintf_irc.h"
#include "s_serv.h"

static int mencap_snotice (struct Client *, struct Client *, int, const char **);
void report_snotice (struct Client *source_p, const char *token, const char *arg);
char *reconstruct_parv(int parc, const char *parv[]);

struct Message snotice_msgtab = {
    "SNOTICE",
    0,
    0,
    0,
    MFLG_SLOW,
    {
	mg_ignore,
	mg_ignore,
	mg_ignore,
	mg_ignore,
	{mencap_snotice, 2},
	mg_ignore
    }
};

mapi_clist_av1 snotice_clist[] = { &snotice_msgtab, NULL };

static int modinit (void)
{
    return 1;
}

static void moddeinit(void)
{
}

DECLARE_MODULE_AV1(snotice, modinit, moddeinit, snotice_clist, NULL, NULL, "$Revision: 1.0 $");

static int mencap_snotice (struct Client *client_p, struct Client *source_p, int parc, const char *parv[])
{
    static char buffer[BUFSIZE];
    char *ptr;
    int cur_len = 0;
    int len, i;

    if (parc < 4)
	report_snotice (source_p, parv[1], parc < 3 ? NULL : parv[2]);

    else
    {
                ptr = buffer;
                cur_len = 0;

                for(i = 2; i < parc; i++)
                {
                        len = strlen(parv[i]) + 1;

                        if((size_t)(cur_len + len) >= sizeof(buffer))
                                return 0;

                        ircsnprintf(ptr, sizeof(buffer) - cur_len, "%s ",
                                 parv[i]);
                        ptr += len;
                        cur_len += len;
                }

	report_snotice (source_p, parv[1], buffer);
    }

    return 0;
}

void report_snotice (struct Client *source_p, const char *token, const char *arg)
{
    if (MyClient(source_p))
	sendto_match_servs(source_p, "*", CAP_ENCAP, NOCAPS, "ENCAP * SNOTICE %s %s", token, arg ? arg : "");

    sendto_realops_flags (UMODE_SERVNOTICE, L_ALL, "SNOTICE %s %s %s", get_oper_name(source_p), token, arg ? arg : "");
}

char *reconstruct_parv(int parc, const char *parv[])
{
  static char tmpbuf[BUFSIZE]; int i;

  strlcpy(tmpbuf, parv[0], BUFSIZE);
  for (i = 1; i < parc; i++)
  {
    strlcat(tmpbuf, " ", BUFSIZE);
    strlcat(tmpbuf, parv[i], BUFSIZE);
  }
  return tmpbuf;
}
